// Ycon -- utility to translate message dropped into window
// into list-structured text.
// 2008/07/27
// Open Source
// Copyright 2008 Peter J. Goodeve

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <Application.h>
#include <Window.h>
#include <Entry.h>
#include <Path.h>
#include <ByteOrder.h>
#include <Clipboard.h>

//#define DPRINTF(str) printf str
#define DPRINTF(str) 

#define ZDPRINTF(str) 

/////////////////////////////////////////////////////////////////


class WMmsgWindow : public BWindow 
{
public:
	WMmsgWindow(BRect frame); 
	bool QuitRequested();
	void MessageReceived(BMessage *msg);
};

WMmsgWindow::WMmsgWindow(BRect frame)
				: BWindow(frame, "Ycon",
					 B_TITLED_WINDOW_LOOK, B_FLOATING_ALL_WINDOW_FEEL, 0 )
{
}

bool WMmsgWindow::QuitRequested()
{
		be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}


void showMsg(BMessage *msg) {
//	msg->PrintToStream();
	char  *name; 
	uint32  type; 
	int32   count;
	uint32 code;
	code = htonl(msg->what);
	printf("['%.4s',\n", &code);
	for(int32 i = 0; 
		msg->GetInfo(B_ANY_TYPE, i, &name, &type, &count) == B_OK; i++ ) {
		code = htonl(type);
		printf("  ['%s', '%.4s', \n", name, &code);
		const void *data;
		ssize_t numbytes;
		for (int32 index=0; index<count; index++) {
			msg->FindData(name, type, index, &data, &numbytes);
			switch (type) {
			case 'CSTR':
				printf("    '%s',\n", data);
				break;
			case 'MIME':	// no terminating null!
				printf("    '%.*s',\n", numbytes, data);
				break;
			case 'LONG':
				printf("    %d,\n", *(int32 *)data);
				break;
			case 'RREF': {
					entry_ref r;
					msg->FindRef(name, index, &r);
					BEntry ent(&r);
					BPath path(&ent);
					printf("    ['%s','%s'],\n",r.name, path.Path());
				}
				break;
			case 'BPNT': {
					BPoint p;
					msg->FindPoint(name, index, &p);
					printf("    ['%.1f','%.1f'],\n",p.x, p.y);
				}
				break;
			default:
				printf("    ['%d bytes'],\n", numbytes);
				break;
			}
		}
		printf("  ],\n");
	}
	printf("]\n");
	fflush(stdout);
}

void WMmsgWindow::MessageReceived(BMessage *msg) {
	switch(msg->what) {
	case B_SIMPLE_DATA:
	case B_MIME_DATA:
			showMsg(msg);
			break;
	case B_PASTE:
			if (be_clipboard->Lock()) {
				showMsg(be_clipboard->Data());
				be_clipboard->Unlock();
			}
			break;
	}
}



thread_id showWindow()
{
	WMmsgWindow		*aWindow;
	BRect			aRect;

	aRect.Set(20,20, 110, 50);
	aWindow = new WMmsgWindow(aRect);
			
	aWindow->Show();
	return aWindow->Thread();

}


//////////////////////////////////////////////////////

int main(int32 argc, char**argv)
{	
	BApplication *myApplication;

	myApplication = new BApplication("application/x-vnd.goodeve-ycon");
	thread_id  wid = showWindow();
	status_t rstat;
	wait_for_thread(wid, &rstat);
	delete(myApplication);
	return(0);
}

